@@ -1,10 +1,7 @@
-#! /usr/bin/python
 
 from twisted.internet import gtk2reactor
 gtk2reactor.install()
 
-from twisted.internet import reactor
-
 import sys, time
 
 import pygtk
