@@ -7,7 +7,7 @@ from twisted.python import log
 from twisted.python.failure import Failure
 from twisted.internet import reactor, defer, error
 
-from buildbot import interfaces
+from buildbot import interfaces, locks
 from buildbot.status.builder import SUCCESS, WARNINGS, FAILURE, EXCEPTION
 from buildbot.status.builder import Results, BuildRequestStatus
 from buildbot.status.progress import BuildProgress
@@ -312,10 +312,17 @@ class Build:
         slavebuilder.slave.updateSlaveStatus(buildStarted=build_status)
 
         # convert all locks into their real forms
-        self.locks = [self.builder.botmaster.getLockByID(l)
-                      for l in self.locks]
+        lock_list = []
+        for access in self.locks:
+            if not isinstance(access, locks.LockAccess):
+                # Buildbot 0.7.7 compability: user did not specify access
+                access = access.defaultAccess()
+            lock = self.builder.botmaster.getLockByID(access.lockid)
+            lock_list.append((lock, access))
+        self.locks = lock_list
         # then narrow SlaveLocks down to the right slave
-        self.locks = [l.getLock(self.slavebuilder) for l in self.locks]
+        self.locks = [(l.getLock(self.slavebuilder), la)
+                       for l, la in self.locks]
         self.remote = slavebuilder.remote
         self.remote.notifyOnDisconnect(self.lostRemote)
         d = self.deferred = defer.Deferred()
@@ -351,15 +358,15 @@ class Build:
         log.msg("acquireLocks(step %s, locks %s)" % (self, self.locks))
         if not self.locks:
             return defer.succeed(None)
-        for lock in self.locks:
-            if not lock.isAvailable():
+        for lock, access in self.locks:
+            if not lock.isAvailable(access):
                 log.msg("Build %s waiting for lock %s" % (self, lock))
-                d = lock.waitUntilMaybeAvailable(self)
+                d = lock.waitUntilMaybeAvailable(self, access)
                 d.addCallback(self.acquireLocks)
                 return d
         # all locks are available, claim them all
-        for lock in self.locks:
-            lock.claim(self)
+        for lock, access in self.locks:
+            lock.claim(self, access)
         return defer.succeed(None)
 
     def _startBuild_2(self, res):
@@ -584,8 +591,8 @@ class Build:
 
     def releaseLocks(self):
         log.msg("releaseLocks(%s): %s" % (self, self.locks))
-        for lock in self.locks:
-            lock.release(self)
+        for lock, access in self.locks:
+            lock.release(self, access)
 
     # IBuildControl
 
