@@ -224,6 +224,14 @@ class ShellCommand:
     KILL = "KILL"
     CHUNK_LIMIT = 128*1024
 
+    # For sending elapsed time:
+    startTime = None
+    elapsedTime = None
+    # I wish we had easy access to CLOCK_MONOTONIC in Python:
+    # http://www.opengroup.org/onlinepubs/000095399/functions/clock_getres.html
+    # Then changes to the system clock during a run wouldn't effect the "elapsed
+    # time" results.
+
     def __init__(self, builder, command,
                  workdir, environ=None,
                  sendStdout=True, sendStderr=True, sendRC=True,
@@ -409,6 +417,7 @@ class ShellCommand:
         # called right after we return, but somehow before connectionMade
         # were called, then kill() would blow up).
         self.process = None
+        self.startTime = time.time()
         p = reactor.spawnProcess(self.pp, argv[0], argv,
                                  self.environ,
                                  self.workdir,
@@ -462,7 +471,8 @@ class ShellCommand:
             self.timer.reset(self.timeout)
 
     def finished(self, sig, rc):
-        log.msg("command finished with signal %s, exit code %s" % (sig,rc))
+        self.elapsedTime = time.time() - self.startTime
+        log.msg("command finished with signal %s, exit code %s, elapsedTime: %0.6f" % (sig,rc,self.elapsedTime))
         for w in self.logFileWatchers:
              # this will send the final updates
             w.stop()
@@ -473,6 +483,7 @@ class ShellCommand:
                 self.sendStatus(
                     {'header': "process killed by signal %d\n" % sig})
             self.sendStatus({'rc': rc})
+        self.sendStatus({'header': "elapsedTime=%0.6f\n" % self.elapsedTime})
         if self.timer:
             self.timer.cancel()
             self.timer = None
@@ -877,6 +888,11 @@ class SlaveFileDownloadCommand(Command):
         self.path = os.path.join(self.builder.basedir,
                                  self.workdir,
                                  os.path.expanduser(self.filename))
+
+        dirname = os.path.dirname(self.path)
+        if not os.path.exists(dirname):
+            os.makedirs(dirname)
+
         try:
             self.fp = open(self.path, 'wb')
             if self.debug:
@@ -1827,14 +1843,14 @@ class Git(SourceBase):
         else:
             head = 'FETCH_HEAD'
 
-        command = ['git-reset', '--hard', head]
+        command = ['git', 'reset', '--hard', head]
         c = ShellCommand(self.builder, command, self._fullSrcdir(),
                          sendRC=False, timeout=self.timeout)
         self.command = c
         return c.start()
 
     def doVCUpdate(self):
-        command = ['git-fetch', self.repourl, self.branch]
+        command = ['git', 'fetch', self.repourl, self.branch]
         self.sendStatus({"header": "fetching branch %s from %s\n"
                                         % (self.branch, self.repourl)})
         c = ShellCommand(self.builder, command, self._fullSrcdir(),
@@ -1850,7 +1866,7 @@ class Git(SourceBase):
 
     def doVCFull(self):
         os.mkdir(self._fullSrcdir())
-        c = ShellCommand(self.builder, ['git-init'], self._fullSrcdir(),
+        c = ShellCommand(self.builder, ['git', 'init'], self._fullSrcdir(),
                          sendRC=False, timeout=self.timeout)
         self.command = c
         d = c.start()
@@ -1859,7 +1875,7 @@ class Git(SourceBase):
         return d
 
     def parseGotRevision(self):
-        command = ['git-rev-parse', 'HEAD']
+        command = ['git', 'rev-parse', 'HEAD']
         c = ShellCommand(self.builder, command, self._fullSrcdir(),
                          sendRC=False, keepStdout=True)
         c.usePTY = False
